"""
Template for Odoo NGINX configuration (HTTP only version).
"""

TEMPLATE = """# Template for Odoo configuration nginx
# 01.04.2025
# upstream server.domain.de {
#     server ip.ip.ip.ip weight=1 fail_timeout=0;
# }

map $http_upgrade $connection_upgrade {
  default upgrade;
  '' close;
}

proxy_cache_path /tmp levels=1:2 keys_zone=my_cache:10m max_size=1g inactive=60m use_temp_path=off;
limit_req_zone $binary_remote_addr$http_x_forwarded_for zone=iprl:16m rate=500r/m;

server {
    listen server.domain.de:80;
    server_name server.domain.de;
    
    # Set max upload size
    client_max_body_size 10G;
    
    access_log /var/log/nginx/server.domain.de-access.log combined buffer=512k flush=1m;
    error_log /var/log/nginx/server.domain.de-error.log;

    # increase proxy buffer to handle some Odoo web requests
    proxy_buffers 16 64k;
    proxy_buffer_size 128k;

    #general proxy settings
    # force timeouts if the backend dies
    proxy_connect_timeout 1200s;
    proxy_send_timeout 1200s;
    proxy_read_timeout 1200s;
    proxy_next_upstream error timeout invalid_header http_500 http_502 http_503;

    # error pages
    error_page 500 502 503 504 /custom_50x.html;
        location = /custom_50x.html {
        root /etc/nginx/html/;
        internal;
    }

    location = /robots.txt {
        add_header Content-Type text/plain;
        return 200 "User-agent: *Disallow: /";
    }

    # security
    include                 nginxconfig.io/security.conf;

    # additional config
    include                 nginxconfig.io/general.conf;

    location / {
        # Add Headers for odoo proxy mode
        proxy_set_header X-Forwarded-Host $http_host;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-Forwarded-Proto $scheme;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_redirect off;
        proxy_pass http://127.0.0.1:{{PORT}};

        add_header Strict-Transport-Security "max-age=31536000; includeSubDomains";
        proxy_cookie_flags session_id samesite=lax secure; 
        #authentication
    }

    # Chat Odoo
    location /websocket {
        proxy_pass http://127.0.0.1:{{POLL_PORT}};

        proxy_set_header Upgrade $http_upgrade;
        proxy_set_header Connection $connection_upgrade;
        proxy_set_header X-Forwarded-Host $http_host;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-Forwarded-Proto $scheme;
        proxy_set_header X-Real-IP $remote_addr;

        add_header Strict-Transport-Security "max-age=31536000; includeSubDomains";
        proxy_cookie_flags session_id samesite=lax secure;
    }

    location ~* /web/static/ {
        proxy_cache_valid 200 60m;
        proxy_buffering    on;
        expires 864000;
        proxy_pass http://127.0.0.1:{{PORT}};
    }
}
""" 


"""
Central file for all NGINX configuration templates.

This module imports all individual template modules and provides 
a single dictionary for accessing them.
"""

from nginx_set_conf.templates.code_server import TEMPLATE as CODE_SERVER_TEMPLATE
from nginx_set_conf.templates.fast_report import TEMPLATE as FAST_REPORT_TEMPLATE
from nginx_set_conf.templates.nextcloud import TEMPLATE as NEXTCLOUD_TEMPLATE
from nginx_set_conf.templates.portainer import TEMPLATE as PORTAINER_TEMPLATE
from nginx_set_conf.templates.odoo_http import TEMPLATE as ODOO_HTTP_TEMPLATE
from nginx_set_conf.templates.odoo_ssl import TEMPLATE as ODOO_SSL_TEMPLATE
from nginx_set_conf.templates.pgadmin import TEMPLATE as PGADMIN_TEMPLATE
from nginx_set_conf.templates.pwa import TEMPLATE as PWA_TEMPLATE
from nginx_set_conf.templates.mailpit import TEMPLATE as MAILPIT_TEMPLATE
from nginx_set_conf.templates.redirect import TEMPLATE as REDIRECT_TEMPLATE
from nginx_set_conf.templates.redirect_ssl import TEMPLATE as REDIRECT_SSL_TEMPLATE
from nginx_set_conf.templates.n8n import TEMPLATE as N8N_TEMPLATE
from nginx_set_conf.templates.kasm import TEMPLATE as KASM_TEMPLATE
from nginx_set_conf.templates.qdrant import TEMPLATE as QDRANT_TEMPLATE

# Replace cache paths to avoid conflicts
def replace_cache_path(template, service_name):
    """Replace the cache path in a template with a unique path based on service name.
    Also replace the zone name to be unique for each service.
    
    Args:
        template (str): The nginx config template
        service_name (str): Name of the service to create unique path
        
    Returns:
        str: Template with updated cache path and zone name
    """
    # Create a unique zone name based on the service
    zone_name = f"{service_name}_cache"
    
    # First replace the cache path
    updated_template = template.replace(
        'proxy_cache_path /tmp', 
        f'proxy_cache_path /var/cache/nginx/{service_name}'
    )
    
    # Then replace the zone name
    updated_template = updated_template.replace(
        'keys_zone=my_cache:',
        f'keys_zone={zone_name}:'
    )
    
    return updated_template

# Weitere Templates hier hinzufügen, wenn sie erstellt wurden

# Dictionary mit allen Templates für einfachen Zugriff
TEMPLATES = {
    "ngx_code_server": replace_cache_path(CODE_SERVER_TEMPLATE, "code_server"),
    "ngx_fast_report": replace_cache_path(FAST_REPORT_TEMPLATE, "fast_report"),
    "ngx_nextcloud": replace_cache_path(NEXTCLOUD_TEMPLATE, "nextcloud"),
    "ngx_portainer": replace_cache_path(PORTAINER_TEMPLATE, "portainer"),
    "ngx_odoo_http": replace_cache_path(ODOO_HTTP_TEMPLATE, "odoo_http"),
    "ngx_odoo_ssl": replace_cache_path(ODOO_SSL_TEMPLATE, "odoo_ssl"),
    "ngx_pgadmin": replace_cache_path(PGADMIN_TEMPLATE, "pgadmin"),
    "ngx_pwa": replace_cache_path(PWA_TEMPLATE, "pwa"),
    "ngx_mailpit": replace_cache_path(MAILPIT_TEMPLATE, "mailpit"),
    "ngx_redirect": replace_cache_path(REDIRECT_TEMPLATE, "redirect"),
    "ngx_redirect_ssl": replace_cache_path(REDIRECT_SSL_TEMPLATE, "redirect_ssl"),
    "ngx_n8n": replace_cache_path(N8N_TEMPLATE, "n8n"),
    "ngx_kasm": replace_cache_path(KASM_TEMPLATE, "kasm"),
    "ngx_qdrant": replace_cache_path(QDRANT_TEMPLATE, "qdrant"),
    # Weitere Templates hier hinzufügen, wenn sie erstellt wurden
}

def get_config_template(config_template_name):
    """
    Get template by name.
    
    Args:
        config_template_name (str): Name of the template to retrieve
        
    Returns:
        str: Template content or empty string if not found
    """
    if config_template_name in TEMPLATES:
        return TEMPLATES[config_template_name]
    else:
        return "" 


"""
nginx-set-conf - Ein Werkzeug zur Verwaltung von Nginx-Konfigurationen
"""

__version__ = '1.3.4'

from . import config_templates, utils

def replace_cache_path(template, service_name):
    """Replace the cache path in a template with a unique path based on service name.
    Also replace the zone name to be unique for each service.
    
    Args:
        template (str): The nginx config template
        service_name (str): Name of the service for unique path and zone
        
    Returns:
        str: Template with updated cache path and zone name
    """
    # Create a unique zone name based on the service
    zone_name = f"{service_name}_cache"
    
    # First replace the cache path
    updated_template = template.replace(
        'proxy_cache_path /tmp', 
        f'proxy_cache_path /var/cache/nginx/{service_name}'
    )
    
    # Then replace the zone name
    updated_template = updated_template.replace(
        'keys_zone=my_cache:',
        f'keys_zone={zone_name}:'
    )
    
    return updated_template
